#ifndef pfodVC820MeterParser_h
#define pfodVC820MeterParser_h
#include "Arduino.h"
/**
pfodVC820MeterParser for Arduino

(c)2016 Forward Computing and Control Pty. Ltd.
www.forward.com.au
This code may be freely used for both private and commercial use.
Provide this copyright is maintained.
*/

class pfodVC820MeterParser {
  public:
    pfodVC820MeterParser(void);
    void setDebugOut(Print *debugOut);
    bool haveReading(); // true if reading valid,
    // if newReading true then only returns true if there is new reading since the last call to haveReading();
    // if returns true then calls to isRelative, isAuto, getType, getAsFloat, getAsStr are valid until next call to haveReading()
    enum {NO_READING = 0, VOLTS_AC = 1, VOLTS_DC = 2, AMPS_AC = 3, AMPS_DC = 4, OHMS = 5, HZ = 6, DUTY_CYCLE_PERCENT = 7, FARADS = 8, DEGS_C = 9}; // measurement types
    enum {NO_SCALING = 0, KILO = 1, MEGA = 2, MILLI = 3, MICRO = 4, NANO = 5}; // measurement scaling

    float getAsFloat(); // used for local calcs in Arduino
    // if isValid()  getAsFloat() ==  atof(getDigits()) * getScaling()
    // else returns 0.0   if not valid

    const char* getAsStr(); // just screen reading with numeric scaling, no units  used for sending plot data

    const char* getDigits(); // just screen reading no scaling or units
    const char* getScalingAsNumberStr(); // e-9, e-6, e-3, ,e3,e6
    const char* getScalingAsStr(); // K,M,m,u,n
    const char* getTypeAsStr();  // V AC, V DC, A AC, A DC, ohms, Hz, %, F, C
    const char* getTypeAsUnicode(); // ohms and degC as unicode others unchanged
    int getType(); // returns measurement type enum above
    float getScaling(); // returns float value of measurement scaling enum above
    bool isRelative();
    bool isAuto();
    bool isValid();
    bool isHold(); // reading held so will not update!!
    bool isDiode();
    bool isOverflow();
    void connect(Stream *input);
    size_t safe_ncat(char* dest, const char* src, size_t spaceRemaining);
    size_t safe_addChar(char* dest, const char c, size_t spaceRemaining);
  private:
    // getByteIdx(byte) returns   zero based index of this byte as determined from the leading nibble
    int getByteIdx(byte b) {
      return (0x0f & (b >> 4)) - 1;
    }
    void clearOutByteArray();
    bool checkFormat();
    uint8_t measurementType;
    uint8_t measurementScaling;
    bool autoSet;
    bool acSet;
    bool relativeSet;
    bool holdSet;
    bool negSet; // true if neg reading
    bool diodeSet;
    bool haveNonZeroDigit;
    float floatReading;
    bool overflowSet;
    const char *emptyStr = "";
    Stream *meterInput;
    Print *debugOut;
    void init();
    const static byte MAX_BYTES = 14;
    uint8_t byteArray[MAX_BYTES];

    const static byte MAX_READING_LEN = 7;
    char reading[MAX_READING_LEN + 1]; // allow for terminating null
    const static byte MAX_STR_READING_LEN = MAX_READING_LEN+4;
    char strReading[MAX_STR_READING_LEN+1];
    const static byte MAX_F_STR = 5;
    char fStr[MAX_F_STR+1];   
    const char* getFstr(const __FlashStringHelper *ifsh, char*rtn);
     
    void parseByte0AutoRS232(byte * barray);
    void parseByte0ACDC(byte* barray);
    void parseHzVA(byte* barray);
    void parseDegC(byte* barray);
    void parseRelOhmFarrad(byte* barray);
    void parseMegaMilliPecent(byte* barray);
    void parseKiloNanoMicro(byte* barray);
    size_t parseByteDigit(byte* barray, char* rtn, size_t spaceRemaining, int digitPos);

};
#endif //pfodVC820MeterParser_h
